/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.util;

/**
 * Factory Class: return StreamReader objects
 */
public class StreamReaderFactory {
    
    private static GzipStreamReader gzipStreamReader;
    private static SimpleStreamReader simpleStreamReader;
    
    /**
     * Default Constructor
     */
    private StreamReaderFactory() {
    }
    
    /**
     * Factory method for object implementig <code>StreamReader</code> interface 
     * @param type is the type of reader to be returned
     * @return StreamReader Object
     */
    public static StreamReader getStreamReader(String type) {
        if (type != null) {
            if (type.equals("gzip")) {
                if (gzipStreamReader==null) {
                    gzipStreamReader = new GzipStreamReader();    
                } 
                return gzipStreamReader;
            } else if (type.equals("deflate")) {
                return null;
            }
            //put here other StreamReaders;
        } else {
            if (simpleStreamReader==null) {
                simpleStreamReader = new SimpleStreamReader();
            }
            return simpleStreamReader;
        }
        return null;
    }
}

