/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package com.funambol.mail;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.microedition.rms.RecordStore;

import jmunit.framework.cldc10.TestCase;
import jmunit.framework.cldc10.TestSuite;

import javax.microedition.rms.RecordStoreException;
import jmunit.framework.cldc10.AssertionFailedException;

import com.funambol.util.Log;

public class FolderTest extends TestCase {

    private RMSStore store = null;
    private Folder folder = null;
    private Message message = null;
    private String folderName1 = "/INBOX";
    private String folderName2 = "/OUTBOX";
    
    public FolderTest() {
        super(9, "FolderTest");

        // Empty record store to begin
        
        try {
            RecordStore.deleteRecordStore(folderName1);
            RecordStore.deleteRecordStore(folderName2);
        }
        catch(Exception e){
            e.printStackTrace();
        }
        
        store = new RMSStore();
        Log.setLogLevel(Log.TRACE);
    }
    
    public void test(int testNumber) throws Throwable {

        switch(testNumber) {
            //creates INBOX and OUTBOX folders
            case 0: testCreateFolders();    break;
            
            //List folders
            case 1: testList();             break;
            
            //put 5 messages into INBOX and 3 messages into OUTBOX
            case 2: testInit();             break;
            
            //Read all Headers of INBOX messages
            case 3: testMessageView();      break;
            
            //Access a message by Message ID and read it
            case 4: testMessageOpen();      break;
            
            //Open all messages
            case 5: testViewAll();          break;
            
            //Delete one message
            case 6: testMessageDelete();    break;
            
            //Delete all message
            case 7: testAllMessageDelete();    break;
            
            //Delete all message
            case 8: makeRoomTest();    break;
            
            default:                        break;
        }
        //reload folder
    }
    
    /**
     * Creates mail Store
     */
    public void testCreateFolders() throws Exception {
        store.createFolder(folderName1);
        store.createFolder(folderName2);
    }

    /**
     * Creates mail Store
     */
    public void testList() throws Exception {
        Folder[] rootFolders = store.list();
    }

    /**
     * Writes messages into INBOX and OUTBOX folders
     * 
     */
    private void testInit() throws Exception {
        
        //INBOX folder
        folder = new Folder(folderName1, store);
        assertEquals(folder.getFullName(), folderName1);
        
        for (int i=0; i<102; i++) {
            
            String content = new String("message_in_content_"+i);
            
            Address from = new Address(Address.FROM, "sender IN " + i, 
                                    "sender_IN_"+ i + "@funambol.com");
            
            Log.debug("From: "+from.toString());

            Address[] to = new Address[3];
            Address[] cc = new Address[3];
            Address[] bcc = new Address[3];
            
            for (int j=0; j<3; j++) {
                
                to[j] = new Address(Address.TO,
                                    "Receiver IN "+j,
                                    "Receiver_IN_"+ j + "@funambol.com");
                cc[j] = new Address(Address.CC,
                                    "Cc_Receiver IN "+j,
                                    "Cc_Receiver_IN_"+ j + "@funambol.com");
                bcc[j] = new Address(Address.BCC,
                                    "Bcc_Receiver IN "+j,
                                    "Bcc_Receiver_IN_"+ j + "@funambol.com");
            }
            
            message = new Message();
            
            message.setBcc(bcc);
            message.setCc(cc);
            message.setFrom(from);
            message.setTo(to);
            
            Log.debug("To: "+message.getHeader(Message.TO));
            Log.debug("Cc: "+message.getHeader(Message.CC));
            Log.debug("Bcc:"+message.getHeader(Message.BCC));

            message.setContent(content);
            message.setSubject("Subject_IN_"+i);
            
            folder.appendMessage(message);
        }
        
        //OUTBOX folder
        folder = new Folder(folderName2, store);
        assertEquals(folder.getFullName(), folderName2);
        
        for (int i=0; i<5; i++) {
            
            String content = new String("message_out_content_"+i);
            
            Address from = new Address(Address.FROM, "sender OUT "+ i, 
                                "sender_OUT_"+ i + "@funambol.com");
            

            StringBuffer tolist = new StringBuffer();
            StringBuffer cclist = new StringBuffer();
            StringBuffer bcclist = new StringBuffer();
            
            for (int j=0; j<3; j++) {
                
                tolist.append("\"Receiver OUT ").append(j)
                  .append("\" <Receiver_OUT_").append(j)
                  .append("@funambol.com>, ");

                cclist.append("\"Cc_Receiver OUT ").append(j)
                  .append("\" <Cc_Receiver_OUT_").append(j)
                  .append("@funambol.com>, ");

                bcclist.append("\"Bcc_Receiver OUT ").append(j)
                   .append("\" <Bcc_Receiver_OUT_").append(j)
                   .append("@funambol.com>, ");
            }
            
            message = new Message();

            message.setFrom(from);

            // Test parse to array and setTo
            Address[] to = Address.parse(Address.TO, tolist.toString());
            message.setTo(to);

            Address[] cc = Address.parse(Address.CC, cclist.toString());
            Address[] bcc = Address.parse(Address.BCC, bcclist.toString());

            // Make a list of different addresses
            Address[] list = new Address[cc.length+bcc.length+1];
            list[0] = new Address(Address.REPLYTO,
                                  "ReplyMe", "reply-me@funambol.com");
            System.arraycopy(cc, 0, list, 1, cc.length);
            System.arraycopy(bcc, 0, list, cc.length+1, bcc.length);

            // Add all the recipients at once
            message.addRecipients(list);
            
            message.setContent(content);
            message.setSubject("Subject_OUT_"+i);
            
            folder.appendMessage(message);
        }
        
    }
    
    /**
     * Read all Headers of INBOX messages
     */
    private void testMessageView() throws Exception {
        Message[] allMessages = store.getMsgHeaders(folderName1);
    }

    /**
     * Access a message by Message ID and read it
     */
    private void testMessageOpen() throws Exception {
        folder.getMessage(1);
        folder.getMessage(2);
        folder.getMessage(3);
        folder.getMessage(4);
        folder.getMessage(5);
        
        
    }

    /**
     * Open all messages
     */
    private void testViewAll() throws Exception {
        for(int i=1; i<=folder.getMessageCount(); i++) {
            folder.getMessage(i);
        }
    }

    /**
     * Delete one message
     */
    private void testMessageDelete() throws Exception {
        folder.deleteMessage(2);
    }

    /**
     * Delete all message
     */
    private void testAllMessageDelete() throws Exception {
        //FIXME: Write a method to delete a given Message Array 
        while(folder.getMessageCount() > 0) {
             folder.deleteMessage(1);
        }
    }

    private void makeRoomTest() throws Exception {
        for (int i=0; i<=102; i++) {
            Message m = new Message();
            folder.appendMessage(message);
            Log.debug("Total Messages in the store: " + folder.getMessageCount());
        }
        Log.debug("Total Messages in the store: " + folder.getMessageCount());
    }

}

